@@ -1,13 +1,14 @@
 /* ebrowse.c --- parsing files for the ebrowse C++ browser
 
    Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-                 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+                 2002, 2003, 2004, 2005, 2006, 2007, 2008
+                 Free Software Foundation, Inc.
 
    This file is part of GNU Emacs.
 
    GNU Emacs is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Emacs is distributed in the hope that it will be useful,
@@ -3691,7 +3692,7 @@ void
 version ()
 {
   printf ("ebrowse %s\n", VERSION);
-  puts ("Copyright (C) 1992-2007 Free Software Foundation, Inc.");
+  puts ("Copyright (C) 2008 Free Software Foundation, Inc.");
   puts ("This program is distributed under the same terms as Emacs.");
   exit (EXIT_SUCCESS);
 }
