@@ -1,7 +1,7 @@
 ;;; cus-start.el --- define customization properties of builtins
 ;;
 ;; Copyright (C) 1997, 1999, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 ;;
 ;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
 ;; Keywords: internal
@@ -10,7 +10,7 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
@@ -42,7 +42,7 @@
 	     (gc-cons-threshold alloc integer)
 	     (garbage-collection-messages alloc boolean)
 	     ;; buffer.c
-	     (mode-line-format modeline sexp) ;Hard to do right.
+	     (mode-line-format mode-line sexp) ;Hard to do right.
 	     (default-major-mode internal function)
 	     (enable-multibyte-characters mule boolean)
 	     (case-fold-search matching boolean)
@@ -98,7 +98,8 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 	     (scroll-down-aggressively windows
 				       (choice (const :tag "off" nil) number)
 				       "21.1")
-	     (line-spacing display (choice (const :tag "none" nil) integer))
+	     (line-spacing display (choice (const :tag "none" nil) integer)
+			   "22.1")
 	     ;; callint.c
 	     (mark-even-if-inactive editing-basics boolean)
 	     ;; callproc.c
@@ -122,10 +123,13 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 				  :value (undecided . undecided)
 				  (coding-system :tag "Decoding")
 				  (coding-system :tag "Encoding"))
-			    (coding-system :tag "Single coding system"
-					   :value undecided)
+			    (coding-system
+			     :tag "Single coding system"
+			     :value undecided
+			     :match (lambda (widget value)
+				      (and value (not (functionp value)))))
 			    (function :value ignore))))
-	     (selection-coding-system mule coding-system)
+	     (selection-coding-system mule coding-system "22.1")
 	     ;; dired.c
 	     (completion-ignored-extensions dired
 					    (repeat (string :format "%v")))
@@ -164,6 +168,7 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 	     ;; fns.c
 	     (use-dialog-box menu boolean "21.1")
 	     (use-file-dialog menu boolean "22.1")
+	     (focus-follows-mouse frames boolean "20.3")
 	     ;; frame.c
 	     (default-frame-alist frames
 	       (repeat (cons :format "%v"
@@ -171,7 +176,8 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 			     (sexp :tag "Value"))))
 	     (mouse-highlight mouse (choice (const :tag "disabled" nil)
 					    (const :tag "always shown" t)
-					    (other :tag "hidden by keypress" 1)))
+					    (other :tag "hidden by keypress" 1))
+			      "22.1")
 	     ;; fringe.c
 	     (overflow-newline-into-fringe fringe boolean)
 	     ;; indent.c
@@ -185,7 +191,7 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 	     (polling-period keyboard integer)
 	     (double-click-time mouse (restricted-sexp
 				       :match-alternatives (integerp 'nil 't)))
-	     (double-click-fuzz mouse integer)
+	     (double-click-fuzz mouse integer "22.1")
 	     (inhibit-local-menu-bar-menus menu boolean)
 	     (help-char keyboard character)
 	     (help-event-list keyboard (repeat (sexp :format "%v")))
@@ -243,9 +249,9 @@ Leaving \"Default\" unchecked is equivalent with specifying a default of
 	     (completion-auto-help minibuffer boolean)
 	     (enable-recursive-minibuffers minibuffer boolean)
 	     (history-length minibuffer
-			     (choice (const :tag "Infinite" t)
-				     integer))
-	     (history-delete-duplicates minibuffer boolean)
+			     (choice (const :tag "Infinite" t) integer)
+			     "22.1")
+	     (history-delete-duplicates minibuffer boolean "22.1")
 	     (minibuffer-prompt-properties
 	      minibuffer
 	      (list
@@ -342,7 +348,7 @@ since it could result in memory overflow and make Emacs crash."
  	      windows (choice
  		       (const :tag "Off (nil)" :value nil)
  		       (const :tag "Full screen (t)" :value t)
- 		       (other :tag "Always" 1)))
+ 		       (other :tag "Always" 1)) "22.1")
 	     (display-buffer-reuse-frames windows boolean "21.1")
 	     ;; xdisp.c
 	     (scroll-step windows integer)
@@ -351,12 +357,12 @@ since it could result in memory overflow and make Emacs crash."
 	     (hscroll-margin windows integer "22.1")
 	     (hscroll-step windows number "22.1")
 	     (truncate-partial-width-windows display boolean)
-	     (mode-line-inverse-video modeline boolean)
-	     (mode-line-in-non-selected-windows modeline boolean "22.1")
+	     (mode-line-inverse-video mode-line boolean)
+	     (mode-line-in-non-selected-windows mode-line boolean "22.1")
 	     (line-number-display-limit display
 					(choice integer
 						(const :tag "No limit" nil)))
-	     (line-number-display-limit-width display integer)
+	     (line-number-display-limit-width display integer "22.1")
 	     (highlight-nonselected-windows display boolean)
 	     (message-log-max debug (choice (const :tag "Disable" nil)
 					    (integer :menu-tag "lines"
@@ -371,7 +377,7 @@ since it could result in memory overflow and make Emacs crash."
 		       (const :tag "Immediate" :value t)
 		       (number :tag "Delay by secs" :value 0.5)) "22.1")
 	     ;; xfaces.c
-	     (scalable-fonts-allowed display boolean)
+	     (scalable-fonts-allowed display boolean "22.1")
 	     ;; xfns.c
 	     (x-bitmap-file-path installation
 				 (repeat (directory :format "%v")))
@@ -462,5 +468,5 @@ since it could result in memory overflow and make Emacs crash."
 (unless purify-flag
   (provide 'cus-start))
 
-;;; arch-tag: 4502730d-bcb3-4f5e-99a3-a86f2d54af60
+;; arch-tag: 4502730d-bcb3-4f5e-99a3-a86f2d54af60
 ;;; cus-start.el ends here
