@@ -2,13 +2,13 @@
 
 /* GNU Emacs site configuration template file.  -*- C -*-
    Copyright (C) 1988, 1993, 1994, 1999, 2000, 2001, 2002, 2003, 2004,
-      2005, 2006, 2007  Free Software Foundation, Inc.
+      2005, 2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
