@@ -582,7 +582,7 @@ _start()
 #endif /* elxsi */
 
 
-#ifdef __sparc__
+#ifdef sparc
 asm (".global __start");
 asm (".text");
 asm ("__start:");
@@ -600,7 +600,7 @@ asm ("	sub	%sp, 24, %sp");
 asm ("	call	__exit");
 asm ("	nop");
 
-#endif /* __sparc__ */
+#endif /* sparc */
 
 #if __FreeBSD__ == 2
 char *__progname;
