@@ -5,3 +5,6 @@
    USUAL-OPSYS="uxpv"  */
 
 #include "delta88k.h"
+
+/* arch-tag: a0ed4f18-ecf2-4d9a-853b-36e0f6b45945
+   (do not change this comment) */
