@@ -1,11 +1,12 @@
 /* machine description file for ibm ps/2 aix386.
-   Copyright (C) 1989 Free Software Foundation, Inc.
+   Copyright (C) 1989, 2001, 2002, 2003, 2004, 2005,
+                 2006, 2007, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 3, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,11 +16,11 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 
-/* The following line tells the configuration script what sort of 
+/* The following line tells the configuration script what sort of
    operating system this machine is likely to run.
    USUAL-OPSYS="note"
 
@@ -29,7 +30,7 @@ NOTE-START
   work with certain new X window managers, and may be suboptimal.
 NOTE-END */
 
-/* Define WORDS_BIG_ENDIAN iff lowest-numbered byte in a word
+/* Define WORDS_BIG_ENDIAN if lowest-numbered byte in a word
    is the most significant byte.  */
 
 #undef WORDS_BIG_ENDIAN
@@ -114,7 +115,6 @@ so disable it for them.  */
 #endif
 
 #define BSTRING
-#define HAVE_VFORK
 #undef  HAVE_TERMIO
 #define HAVE_TERMIOS
 
@@ -188,25 +188,15 @@ so disable it for them.  */
 
 /* Here override various assumptions in ymakefile */
 
-/* Define C_ALLOCA if this machine does not support a true alloca
-   and the one written in C should be used instead.
-   Define HAVE_ALLOCA to say that the system provides a properly
-   working alloca function and it should be used.
-   Define neither one if an assembler-language alloca
-   in the file alloca.s should be used.  */
-
 #ifdef __GNUC__
-#define HAVE_ALLOCA
 #define alloca(n) __builtin_alloca(n)
 #if __GNUC__ < 2
-#define LIB_STANDARD /usr/local/lib/gcc-gnulib -lbsd -lrts -lc 
+#define LIB_STANDARD /usr/local/lib/gcc-gnulib -lbsd -lrts -lc
 #endif
 /* -g fails to work, so it is omitted.  */
 /* tranle says that -fstrength-reduce does not help.  */
 #define C_DEBUG_SWITCH
 #else
-#define C_ALLOCA
-#define STACK_DIRECTION -1 /* tell alloca.c which way it grows */
 #define LIBS_MACHINE -lbsd -lrts
 #endif
 
@@ -241,3 +231,6 @@ so disable it for them.  */
 #define LD_SWITCH_MACHINE -shlib
 #endif
 #endif
+
+/* arch-tag: 2e7f44df-6a61-4a47-aa53-f7961bfeff11
+   (do not change this comment) */
